/*
   FILNAME:    

   PURPOSE:   

   CREATED BY:    Per Ghosh
   ENVIRONMENT:	Visual C++


INFO:
*/

#include "stdafx.h"

#include "PGStorageManager.h"
#include "PGStream.h"

#include "PGStorage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNAMIC(CPGStorage, CObject)

HRESULT CPGStorage::m_hResult = 0;

/************************************************
   *   Constructor/Destructor 
   *
 * * *
  ***
   *
*/

//-------------------------------------------------------------------------------------------------
CPGStorage::CPGStorage() :
   m_pIStorage(NULL), m_pwName(NULL), m_pStorageParent(NULL)
{
}

//-------------------------------------------------------------------------------------------------
   CPGStorage::CPGStorage( LPCSTR pszName ) :
   m_pIStorage(NULL), m_pStorageParent(NULL)
{
   m_pwName = new WCHAR[strlen(pszName) + 1];

   ::MultiByteToWideChar( CP_ACP, 
                          MB_PRECOMPOSED, 
                          pszName, 
                          -1, 
                          m_pwName, 
                          strlen(pszName) + 1 );
}

//-------------------------------------------------------------------------------------------------
CPGStorage::CPGStorage( const WCHAR* pwName ) :
   m_pIStorage(NULL), m_pStorageParent(NULL)
{
   m_pwName = new WCHAR[wcslen(pwName) + 1];
   wcscpy( m_pwName, pwName );
}

//-------------------------------------------------------------------------------------------------
CPGStorage::~CPGStorage() 
{
   int iCounter;

   iCounter = m_arrayToStorage.GetSize();
   while( iCounter )
   {
      iCounter--;
      delete m_arrayToStorage[iCounter];
   }

   iCounter = m_arrayToStream.GetSize();
   while( iCounter )
   {
      iCounter--;
      delete m_arrayToStream[iCounter];
   }

   if( m_pwName )    delete m_pwName;

   if( m_pIStorage ) m_pIStorage->Release();
}




/************************************************
   *   Functions 
   *
 * * *
  ***
   *
*/

/*-------------------------------------------------------------------------------------------------
Name:	   CreateStorage()
Type:    Function
Purpose: Open a storage

Param:   pwName = name of the storage object to open
         dwFlags = access mode to use when creating the storage 

Return:  CPGStorage*, Pointer to CPGStorage if created otherwise NULL
*/
CPGStorage* CPGStorage::CreateStorage( WCHAR* pwName, DWORD dwFlags /*=STORAGE_T_RW_SE*/)
{
   ASSERT( wcslen(pwName) < 33 );

   CPGStorage* pStorage = new CPGStorage( pwName );

   // ***
   // Create storage
   // ***
   CPGStorage::m_hResult = m_pIStorage->CreateStorage( pwName, 
                                                       dwFlags,  
                                                       0, 
                                                       0, 
                                                       &pStorage->m_pIStorage ); 

   if( FAILED(CPGStorage::m_hResult) )
   {
      delete pStorage;
      return NULL;
   }

   pStorage->m_pStorageParent = this;  // Set parent storage
   m_arrayToStorage.Add( pStorage );   // Add to array

   return pStorage;

}

/*-------------------------------------------------------------------------------------------------
Name:	   OpenStorage()
Type:    Function
Purpose: Open a storage

Param:   pwName = name of the storage object to open
         dwFlags = access mode to use when opening the storage 

Return:  CPGStorage*, Pointer to CPGStorage if found otherwise NULL
*/
CPGStorage* CPGStorage::OpenStorage( WCHAR* pwName, DWORD dwFlags /*=STORAGE_T_RW_SE*/ )
{
   ASSERT( wcslen(pwName) < 33 );

   CPGStorage* pStorage = new CPGStorage( pwName );

   // ***
   // Open storage
   // ***
   CPGStorage::m_hResult = m_pIStorage->OpenStorage( pwName, 
                                                     NULL, 
                                                     dwFlags, 
                                                     NULL, 
                                                     0, 
                                                     &pStorage->m_pIStorage ); 

   if( FAILED(CPGStorage::m_hResult) )
   {
      delete pStorage;
      return NULL;
   }

   pStorage->m_pStorageParent = this;  // Set parent storage
   m_arrayToStorage.Add( pStorage );   // Add to array

   return pStorage;
}

/*-------------------------------------------------------------------------------------------------
Name:	   CreateStream()
Type:    Function
Purpose: Create a stream in storage

Param:   pwName = name of stream
         dwFlags =  Specifies the access mode to use when creating the stream

Return:  CPGStream*, Pointer to CPGStream if found otherwise NULL
*/
CPGStream* CPGStorage::CreateStream( WCHAR* pwName, DWORD dwFlags /*=STORAGE_D_RW_SE*/)
{
   ASSERT( wcslen(pwName) < 33 );

   IStream* pIStream; 

   // ***
   // Create stream
   // ***
   CPGStorage::m_hResult = m_pIStorage->CreateStream( pwName, dwFlags, 0, 0, &pIStream ); 

   if( FAILED(CPGStorage::m_hResult) )
   {
      return NULL;         
   }

   CPGStream* pStream = new CPGStream( pIStream ); // Create new streamobject

   m_arrayToStream.Add( pStream );                 // Add to array
   m_pstreamActive = pStream;                      // Set to active stream               

   return pStream;
}

/*-------------------------------------------------------------------------------------------------
Name:	   OpenStream()
Type:    function
Purpose: Open a stream in storage

Param:   pwName = name of stream
         dwFlags = Specifies the access mode to use when opening the stream

Return:  CPGStream*, Pointer to CPGStream if found otherwise NULL
*/
CPGStream* CPGStorage::OpenStream( WCHAR* pwName, DWORD dwFlags /*=STORAGE_D_RW_SE*/ )
{
   ASSERT( wcslen(pwName) < 33 );

   IStream* pIStream; 

   // ***
   // Open Stream
   // ***
   CPGStorage::m_hResult = m_pIStorage->OpenStream( pwName, NULL, dwFlags, 0, &pIStream ); 

   if( FAILED(CPGStorage::m_hResult) )
   {
      return NULL;
   }

   CPGStream* pStream = new CPGStream( pIStream ); // Create new streamobject

   m_arrayToStream.Add( pStream );                 // Add to array
   m_pstreamActive = pStream;                      // Set to active stream               

   return pStream;
}

/*-------------------------------------------------------------------------------------------------
Name:	   Dir()
Type:    Function
Purpose: Enumrate the elements in storage

Param:   ptrarrayToSTATSTG = array that all STATSTG-structures are put
         dwType = type of elements that should be put in array (if 0 all goes into array).
         dwMaxElements = max elements that are listed

Return:  int, number of elements put in array
*/
int CPGStorage::Dir( CPtrArray& ptrarrayToSTATSTG, DWORD dwType, DWORD dwMaxElements )
{
   ASSERT( m_pIStorage );

   UINT  uCounter = 0;          // Elements that has been listed
   ULONG uElementFetched = 1;   // Number of elements that was fetched in IEnumSTATSTG::Next()
   BOOL  bAddToArray = TRUE;    // Marks if element should be added to array
   BOOL  bSTATSTGAdded;

   IEnumSTATSTG* pIEnumSTATSTG; // Interface for enumrating elements in storage
   STATSTG*      pstatstg;      // Pointer to STATSTG-structure
   
   // Get IEnumSTATSTG interface
   CPGStorage::m_hResult = m_pIStorage->EnumElements( 0, NULL, 0, &pIEnumSTATSTG );

   if( FAILED(CPGStorage::m_hResult) )
   {
      return 0;
   }

   // ***
   // Get the elements in storage
   // ***
   while( (uCounter < dwMaxElements) && uElementFetched )
   {
      if( bAddToArray )
      {
         pstatstg = new STATSTG;
         bSTATSTGAdded = FALSE;
      }

      bAddToArray = TRUE;
      pIEnumSTATSTG->Next( 1, pstatstg, &uElementFetched );

      if( dwType )
      {
         bAddToArray = (pstatstg->type == dwType) ? TRUE : FALSE;
      }

      if( uElementFetched == 0 )
      {
         bAddToArray = FALSE;
      }

      if( bAddToArray )
      {
         ptrarrayToSTATSTG.Add( pstatstg );
         bSTATSTGAdded = TRUE;
      }

      uCounter++;
   }

   pIEnumSTATSTG->Release();

   // if did not add the last "pstatstg" then delete it.
   if( bSTATSTGAdded == FALSE ) delete pstatstg;

   return ptrarrayToSTATSTG.GetSize();
}

/*-------------------------------------------------------------------------------------------------
Name:	 
Type:    
Purpose: 

Param:

Return:
*/
BOOL CPGStorage::Commit( DWORD dwFlags /*=STGC_DEFAULT*/ )
{
   BOOL bReturn = TRUE;

   CPGStorage::m_hResult = m_pIStorage->Commit( dwFlags );

   if( FAILED(CPGStorage::m_hResult) )
   {
      return FALSE;
   }

   if( m_pStorageParent )
   {
      bReturn = m_pStorageParent->Commit( dwFlags );
   }

   return bReturn;
}





/************************************************
   *   Debug
   *
 * * *
  ***
   *
*/


#ifdef _DEBUG
void CPGStorage::AssertValid() const
{
	CObject::AssertValid();
}

void CPGStorage::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
}
#endif //_DEBUG




