/*
   FILNAME:       PGStream.cpp    

   PURPOSE:   

   CREATED BY:    Per Ghosh
   ENVIRONMENT:   Visual C++


INFO:
*/

#include "stdafx.h"

#include "PGStream.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPGStream

IMPLEMENT_DYNAMIC(CPGStream, CFile)

HRESULT CPGStream::m_hResult = 0;

#define PGX_ERR_WRITE      "Fail to write"
#define PGX_ERR_READ       "Fail to read"
#define PGX_ERR_SEEK       "Fail to seek"

/************************************************
   *   Constructor/Destructor 
   *
 * * *
  ***
   *
*/

//-------------------------------------------------------------------------------------------------
CPGStream::CPGStream() :
   m_pIStream(NULL), m_pwName(NULL)
{
}

//-------------------------------------------------------------------------------------------------
CPGStream::CPGStream( IStream* pStream )
{
	ASSERT(pStream != NULL);

	STATSTG statstg;

   m_pIStream = pStream;                                    // Set m_pIStream

   // Get inforamtion about stream
   CPGStream::m_hResult = m_pIStream->Stat(&statstg, STATFLAG_DEFAULT);

   if( ! FAILED(CPGStream::m_hResult) )
	{
		if(statstg.pwcsName != NULL)
		{
         // *** Copy name ***
         m_pwName = new WCHAR[wcslen(statstg.pwcsName) + 1];
         wcscpy( m_pwName, statstg.pwcsName );

         ::CoTaskMemFree(statstg.pwcsName);                 // Free memory
      }
      else // (statstg.pwcsName == NULL)
      {
         m_pwName = NULL;
      }
   }
}

//-------------------------------------------------------------------------------------------------
CPGStream::CPGStream( IStream* pStream, const WCHAR* pwName )
{
	ASSERT(pStream != NULL);
#ifdef _DEBUG
	STATSTG statstg;
   if( m_pIStream->Stat(&statstg, STATFLAG_DEFAULT) == S_OK )
   {
      ASSERT( wcscmp( statstg.pwcsName, pwName ) == 0 );
      ::CoTaskMemFree(statstg.pwcsName);
   }
#endif

   m_pIStream = pStream;

   m_pwName = new WCHAR[wcslen(pwName) + 1];
   wcscpy( m_pwName, pwName );
}

//-------------------------------------------------------------------------------------------------
CPGStream::~CPGStream() 
{
   if( m_pIStream ) m_pIStream->Release();

   if( m_pwName ) delete m_pwName;
}

/************************************************
   *   Virtual functions
   *
 * * *
  ***
   *
*/

/*-------------------------------------------------------------------------------------------------
Name:	   Seek()
Type:    viritual
Purpose: Reposition the pointer in stream

Param:   lOffset = Number of bytes to move the pointer
         uFromPos = number of bytes thar are going to be read

Return:  LONG, on success the new byte offset from the beginning of the file, otherwise undefined

Except:  Throws a char* exception on error
*/
LONG CPGStream::Seek(LONG lOffset, UINT uFromPos)
{
	ASSERT_VALID(this);
	ASSERT(m_pIStream != NULL);

	ASSERT(STREAM_SEEK_SET == begin);
	ASSERT(STREAM_SEEK_CUR == current);
	ASSERT(STREAM_SEEK_END == end);

   ASSERT((uFromPos==STREAM_SEEK_SET)||(uFromPos==STREAM_SEEK_CUR)||(uFromPos==STREAM_SEEK_END));

	ULARGE_INTEGER liNewPosition;
	LARGE_INTEGER liOffset;
	
   LISet32(liOffset, lOffset);

   // ***
   // Seek in stream
   // ***
	CPGStream::m_hResult = m_pIStream->Seek(liOffset, uFromPos, &liNewPosition);

   if( FAILED(CPGStream::m_hResult) )
   {
		throw PGX_ERR_SEEK;
   }

   // Does only support 32-bit searches
	ASSERT(liNewPosition.HighPart == 0);

   return liNewPosition.LowPart;
}

/*-------------------------------------------------------------------------------------------------
Name:	   GetPosition()
Type:    virtual
Purpose: Obtains the current value of the file pointer, can be used in subsequent calls to Seek

Return:  DWORD, The file pointer

Except:  Throws a long-variable
*/
DWORD CPGStream::GetPosition() const
{
	ASSERT(m_pIStream != NULL);

	ULARGE_INTEGER liPosition;
	LARGE_INTEGER  liZero; 
   LISet32(liZero, 0);

   // ***
   // Get position
   // ***
	CPGStream::m_hResult = m_pIStream->Seek(liZero, STREAM_SEEK_CUR, &liPosition);

   if( FAILED(CPGStream::m_hResult) )
   {
		throw CPGStream::m_hResult;
   }

   // Does only support 32-bit searches
	ASSERT(liPosition.HighPart == 0);

   return liPosition.LowPart;
}



/*-------------------------------------------------------------------------------------------------
Name:	   Read()
Type:    viritual
Purpose: Read bytes from a stream

Param:   pBuffer = pointer to buffer that bytes should be read in to
         uCount = number of bytes thar are going to be read

Return:  UINT, number of bytes read

Except:  Throws a char* exception on error
*/
UINT CPGStream::Read(void* pBuffer, UINT uCount)
{
	ASSERT_VALID(this);
	ASSERT(m_pIStream != NULL);

	DWORD dwBytesRead;

	if (uCount == 0)
		return 0;

	ASSERT(AfxIsValidAddress(pBuffer, uCount));

   // ***
	// read from stream
   // ***
	CPGStream::m_hResult = m_pIStream->Read(pBuffer, uCount, &dwBytesRead);

   if( FAILED(CPGStream::m_hResult) )
   {
		throw PGX_ERR_READ;
   }

	// return bytes read
	return (UINT)dwBytesRead;
}

/*-------------------------------------------------------------------------------------------------
Name:	   Write()
Type:    virtual 
Purpose: Write a specified number of bytes into stream

Param:   pBuffer = pointer to bytes that should be written
         uCount = number of bytes that should be written

Except:  Throws a char* exception on error
*/
void CPGStream::Write(const void* pBuffer, UINT uCount)
{
	ASSERT_VALID(this);
	ASSERT(m_pIStream != NULL);

   DWORD dwBytesWritten;

	if (uCount == 0)
		return;

	ASSERT(AfxIsValidAddress(pBuffer, uCount, FALSE));

   // ***
	// write to stream
   // ***
	CPGStream::m_hResult = m_pIStream->Write(pBuffer, uCount, &dwBytesWritten);

   if( FAILED(CPGStream::m_hResult) )
   {
		throw PGX_ERR_WRITE;
   }

	// if no error, all bytes should have been written
	ASSERT((UINT)dwBytesWritten == uCount);
}

/*-------------------------------------------------------------------------------------------------
Name:	   Close()
Type:    virtual
Purpose: Close a stream
*/
void CPGStream::Close()
{
	ASSERT_VALID(this);
	ASSERT(m_pIStream != NULL);

   m_pIStream->Release();
   m_pIStream = NULL;

   if( m_pwName )
   {
      delete m_pwName;
      m_pwName = NULL;
   }
}

/************************************************
   *   Functions 
   *
 * * *
  ***
   *
*/

/*-------------------------------------------------------------------------------------------------
Name:	   CreateMemoryStream()
Type:    Function
Purpose: Create a stream in memory

Return:  BOOL, TRUE on success otherwise FALSE
*/
BOOL CPGStream::CreateMemoryStream()
{
   CPGStream::m_hResult = ::CreateStreamOnHGlobal(NULL, TRUE, &m_pIStream);
	if ( FAILED(CPGStream::m_hResult) )
   {
      m_pIStream = NULL;
      return FALSE;
   }

   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
Name:	   CreateStream()
Type:    Function
Purpose: Create a stream

Param:   pStorage = pointer to storage where stream is located
         pwName   = name of the stream which are going to be opened
         dwFlags  = (look in some OLE-manual)

Return:  BOOL, TRUE on success otherwise FALSE
*/
BOOL CPGStream::CreateStream( IStorage* pStorage, const WCHAR* pwName, DWORD dwFlags )
{
   ASSERT(pStorage != NULL);
	ASSERT(m_pIStream == NULL);
   ASSERT(m_pwName == NULL);

   m_pwName = new WCHAR[wcslen(pwName) + 1];
   wcscpy( m_pwName, pwName );

	CPGStream::m_hResult = pStorage->CreateStream( m_pwName, dwFlags, 0, 0, &m_pIStream );

	if ( FAILED(CPGStream::m_hResult) )
   {
      m_pIStream = NULL;
      delete m_pwName;
      m_pwName = NULL;
      return FALSE;
   }

   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
Name:	   OpenStream()
Type:    function
Purpose: Open a stream

Param:   pStorage = pointer to storage where stream is located
         pwName   = name of the stream which are going to be opened
         dwFlags  = (look in some OLE-manual)

Return:  BOOL, TRUE on success otherwise FALSE
*/
BOOL CPGStream::OpenStream( IStorage* pStorage, const WCHAR* pwName, DWORD dwFlags )
{
   ASSERT(pStorage != NULL);
	ASSERT(m_pIStream == NULL);
   ASSERT(m_pwName == NULL);

   // ***
   // Open stream
   // ***
	CPGStream::m_hResult = pStorage->OpenStream( pwName, NULL, dwFlags, 0, &m_pIStream );

	if ( FAILED(CPGStream::m_hResult) )
   {
      m_pIStream = NULL;
      return FALSE;
   }

   // *** Copy name ***
   m_pwName = new WCHAR[wcslen(pwName) + 1];
   wcscpy( m_pwName, pwName );

   return TRUE;
}

/************************************************
   *   Debug
   *
 * * *
  ***
   *
*/

#ifdef _DEBUG
void CPGStream::AssertValid() const
{
	CFile::AssertValid();
}

void CPGStream::Dump(CDumpContext& dc) const
{
	CFile::Dump(dc);
   if( m_pwName )
   {
      dc << "Stream name is \"" << m_pwName << "\"\n";
   }

   dc << "Last result on streamoperation was (" << CPGStream::m_hResult << ")\n";
}
#endif //_DEBUG




